@@ -1,23 +1,47 @@
-//---------------------------------------------------------------------------
-//
-// amor.cpp
-//
-// Copyright (c) 1999 Martin R. Jones <mjones@kde.org>
-//
+/* amor.cpp
+**
+** Copyright (c) 1999 Martin R. Jones <mjones@kde.org>
+**
+*/
 
+/*
+** This program is free software; you can redistribute it and/or modify
+** it under the terms of the GNU General Public License as published by
+** the Free Software Foundation; either version 2 of the License, or
+** (at your option) any later version.
+**
+** This program is distributed in the hope that it will be useful,
+** but WITHOUT ANY WARRANTY; without even the implied warranty of
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+** GNU General Public License for more details.
+**
+** You should have received a copy of the GNU General Public License
+** along with this program in a file called COPYING; if not, write to
+** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
+** MA 02111-1307, USA.
+*/
+
+/*
+** Bug reports and questions can be sent to kde-devel@kde.org
+*/
 #include <stdlib.h>
 #include <unistd.h>
 #include <time.h>
 
+#include <kdebug.h>
+
 #include <kpopupmenu.h>
 #include <qtimer.h>
 #include <qcursor.h>
+#include <qvaluelist.h>
 
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kstartupinfo.h>
 #include <kwin.h>
 #include <kwinmodule.h>
+#include <kstandarddirs.h>
+#include <khelpmenu.h>
 #include <kiconloader.h>
 
 #include "amor.h"
@@ -30,12 +54,16 @@
 #include <X11/Xlib.h>
 #include <kdebug.h>
 
+// #define DEBUG_AMOR
+
 #define SLEEP_TIMEOUT   180     // Animation sleeps after SLEEP_TIMEOUT seconds
                                 // of mouse inactivity.
 #define TIPS_FILE       "tips"  // Display tips in TIP_FILE-LANG, e.g "tips-en"
 #define TIP_FREQUENCY   20      // Frequency tips are displayed small == more
                                 // often.
 
+#define BUBBLE_TIME_STEP 250
+
 // Standard animation groups
 #define ANIM_BASE       "Base"
 #define ANIM_NORMAL     "Sequences"
@@ -46,7 +74,50 @@
 #define ANIM_WAKE       "Wake"
 
 //---------------------------------------------------------------------------
+// QueueItem
+// Constructor
 //
+
+QueueItem::QueueItem(itemType ty, QString te, int ti)
+{
+    // if the time field was not given, calculate one based on the type 
+    // and length of the item
+    int effectiveLength = 0, nesting = 0;
+
+    // discard html code from the lenght count
+    for (unsigned int i = 0; i < te.length(); i++)
+    {
+	if (te[i] == '<')	nesting++;
+	else if (te[i] == '>')	nesting--;
+	else if (!nesting)	effectiveLength++;
+    }
+    if (nesting) // malformed html
+    {
+#ifdef DEBUG_AMOR
+	kdDebug(10000) << "QueueItem::QueueItem(): Malformed HTML!" << endl;
+#endif
+	effectiveLength = te.length();
+    }
+
+    if (ti == -1)
+    {
+	switch (ty)  {
+	    case Talk : // shorter times
+			ti = 1500 + 45 * effectiveLength;
+			break;
+	    case Tip  : // longer times
+			ti = 4000 + 30 * effectiveLength;
+			break;
+	}
+    }
+
+    iType = ty;
+    iText = te;
+    iTime = ti;
+}
+
+//---------------------------------------------------------------------------
+// AMOR
 // Constructor
 //
 Amor::Amor() : DCOPObject( "AmorIface" ), QObject()
@@ -71,8 +142,8 @@ Amor::Amor() : DCOPObject( "AmorIface" ), QObject()
                 this, SLOT(slotWindowRemove(WId)));
         connect(mWin, SIGNAL(stackingOrderChanged()),
                 this, SLOT(slotStackingChanged()));
-        connect(mWin, SIGNAL(windowChanged(WId)),
-                this, SLOT(slotWindowChange(WId)));
+        connect(mWin, SIGNAL(windowChanged(WId, const unsigned long *)),
+                this, SLOT(slotWindowChange(WId, const unsigned long *)));
         connect(mWin, SIGNAL(currentDesktopChanged(int)),
                 this, SLOT(slotDesktopChange(int)));
 
@@ -88,6 +159,9 @@ Amor::Amor() : DCOPObject( "AmorIface" ), QObject()
 
         mStackTimer = new QTimer(this);
         connect(mStackTimer, SIGNAL(timeout()), SLOT(restack()));
+    
+	mBubbleTimer = new QTimer(this);
+	connect(mBubbleTimer, SIGNAL(timeout()), SLOT(slotBubbleTimeout()));
 
         time(&mActiveTime);
         mCursPos = QCursor::pos();
@@ -111,7 +185,9 @@ Amor::Amor() : DCOPObject( "AmorIface" ), QObject()
 	else
 		kdDebug(10000) << "attached dcop signals..." << endl;
 
-        KStartupInfo::appStarted();
+	mTipsQueue.setAutoDelete(true);
+
+	KStartupInfo::appStarted();
     }
     else
     {
@@ -132,59 +208,90 @@ Amor::~Amor()
 
 void Amor::screenSaverStopped()
 {
+#ifdef DEBUG_AMOR
     kdDebug(10000)<<"void Amor::screenSaverStopped() \n";
+#endif
+
     mAmor->show();
     mForceHideAmorWidget = false;
+
+    mTimer->start(0, true);
 }
 
 void Amor::screenSaverStarted()
 {
+#ifdef DEBUG_AMOR
     kdDebug(10000)<<"void Amor::screenSaverStarted() \n";
+#endif
+
     mAmor->hide();
     mTimer->stop();
     mForceHideAmorWidget = true;
+
+    // GP: hide the bubble (if there's any) leaving any current message in the queue
+    hideBubble();
 }
 
 //---------------------------------------------------------------------------
 //
 void Amor::showTip( QString tip )
 {
-    if ( mConfig.mAppTips )
+    if (mTipsQueue.count() < 5 && !mForceHideAmorWidget) // start dropping tips if the queue is too long
+        mTipsQueue.enqueue(new QueueItem(QueueItem::Tip, tip));
+
+    if (mState == Sleeping)
     {
-	if ( mCurrAnim == mBaseAnim || mCurrAnim->frameNum() == 0 )
-	    showBubble( tip );
-	else
-	    mTipText = tip;
+	selectAnimation(Waking);	// Set waking immediatedly
+	mTimer->start(0, true);
     }
 }
 
+
+void Amor::showMessage( QString message )
+{
+    showMessage(message, -1);
+}
+
+void Amor::showMessage( QString message , int msec )
+{
+    // FIXME: What should be done about messages and tips while the screensaver is on?
+    if (mForceHideAmorWidget) return; // do not show messages sent while in the screensaver
+
+    mTipsQueue.enqueue(new QueueItem(QueueItem::Talk, message, msec));
+
+    if (mState == Sleeping)
+    {
+	selectAnimation(Waking);	// Set waking immediatedly
+	mTimer->start(0, true);
+    }
+}
+
+
 //---------------------------------------------------------------------------
 //
 // Clear existing theme and reload configuration
 //
 void Amor::reset()
 {
-    mTimer->stop();
+    hideBubble();
+
+    mAmor->setPixmap(0L);	// get rid of your old copy of the pixmap
 
     AmorPixmapManager::manager()->reset();
     mTips.reset();
-    delete mAmor;
+
+//    mTipsQueue.clear();	Why had I chosen to clean the tips queue? insane!
 
     readConfig();
 
-    mTargetWin  = 0;
-    mNextTarget = 0;
     mCurrAnim   = mBaseAnim;
     mPosition   = mCurrAnim->hotspot().x();
     mState      = Normal;
 
-    mAmor = new AmorWidget();
-    mForceHideAmorWidget = false;
-    connect(mAmor, SIGNAL(mouseClicked(const QPoint &)),
-                    SLOT(slotMouseClicked(const QPoint &)));
-    connect(mAmor, SIGNAL(dragged(const QPoint &, bool)),
-		    SLOT(slotWidgetDragged(const QPoint &, bool)));
-    mAmor->resize(mTheme.maximumSize());
+    mAmor->resize(mTheme.maximumSize()); 
+    mCurrAnim->reset();
+
+    mTimer->start(0, true);
 }
 
 //---------------------------------------------------------------------------
@@ -201,6 +308,14 @@ bool Amor::readConfig()
         mTips.setFile(TIPS_FILE);
     }
 
+    // Select a random theme if user requested it
+	if (mConfig.mRandomTheme)
+	{
+		QStringList files(KGlobal::dirs()->findAllResources("appdata", "*rc"));
+		int randomTheme = kapp->random() % files.count();
+		mConfig.mTheme = (QString)*files.at(randomTheme);
+	}
+	
     // read selected theme
     if (!mTheme.setTheme(mConfig.mTheme))
     {
@@ -242,10 +357,14 @@ bool Amor::readConfig()
 //
 // Show the bubble text
 //
-void Amor::showBubble(const QString& msg)
+void Amor::showBubble()
 {
-    if (!msg.isNull())
+    if (!mTipsQueue.isEmpty())
     {
+#ifdef DEBUG_AMOR
+    kdDebug(10000) << "Amor::showBubble(): Displaying tips bubble." << endl;
+#endif
+
         if (!mBubble)
         {
             mBubble = new AmorBubble;
@@ -253,7 +372,10 @@ void Amor::showBubble(const QString& msg)
 
         mBubble->setOrigin(mAmor->x()+mAmor->width()/2,
                            mAmor->y()+mAmor->height()/2);
-        mBubble->setMessage(msg);
+        mBubble->setMessage(mTipsQueue.head()->text());
+
+//	mBubbleTimer->start(mTipsQueue.head()->time(), true);
+	mBubbleTimer->start(BUBBLE_TIME_STEP, true);
     }
 }
 
@@ -261,10 +383,29 @@ void Amor::showBubble(const QString& msg)
 //
 // Hide the bubble text if visible
 //
-void Amor::hideBubble()
+void Amor::hideBubble(bool forceDequeue)
 {
     if (mBubble)
     {
+#ifdef DEBUG_AMOR
+    kdDebug(10000) << "Amor::hideBubble(): Hiding tips bubble" << endl;
+#endif
+
+        // GP: stop mBubbleTimer to avoid deleting the first element, just in case we are changing windows
+	// or something before the tip was shown long enough
+        mBubbleTimer->stop();
+
+	// GP: the first message on the queue should be taken off for a 
+	// number of reasons: a) forceDequeue == true, only when called 
+	// from slotBubbleTimeout; b) the bubble is not visible ; c) 
+	// the bubble is visible, but there's Tip being displayed. The 
+	// latter is to keep backwards compatibility and because 
+	// carrying around a tip bubble when switching windows quickly is really 
+	// annoyying
+	if (forceDequeue || !mBubble->isVisible() || 
+	    (mTipsQueue.head()->type() == QueueItem::Tip)) /* there's always an item in the queue here */
+	    mTipsQueue.dequeue();
+
         delete mBubble;
         mBubble = 0;
     }
@@ -292,6 +433,25 @@ void Amor::selectAnimation(State state)
             if (mTargetWin != None)
             {
                 mTargetRect = KWin::windowInfo(mTargetWin).frameGeometry();
+
+		// if the animation falls outside of the working area, 
+		// then relocate it so that is inside the desktop again
+		QRect desktopArea = mWin->workArea();
+		mInDesktopBottom = false;
+
+		if (mTargetRect.y() - mCurrAnim->hotspot().y() + mConfig.mOffset <
+		    desktopArea.y())
+		{
+		    // relocate the animation at the bottom of the screen
+		    mTargetRect = QRect(desktopArea.x(), 
+				  desktopArea.y() + desktopArea.height(),
+				  desktopArea.width(), 0);
+
+		    // we'll relocate the animation in the desktop 
+		    // frame, so do not add the offset to its vertical position
+		    mInDesktopBottom = true;
+		}
+
 		if ( mTheme.isStatic() )
 		{
 		    if ( mConfig.mStaticPos < 0 )
@@ -327,6 +487,7 @@ void Amor::selectAnimation(State state)
                 mTimer->stop();
             }
             mAmor->hide();
+	    
             restack();
             mState = Normal;
             break;
@@ -351,7 +512,7 @@ void Amor::selectAnimation(State state)
             // is not the base, otherwise select the base.  This makes us
             // alternate between the base animation and a random
             // animination.
-            if (mCurrAnim == mBaseAnim && (!mBubble || !mBubble->isVisible() ) )
+	    if (mCurrAnim == mBaseAnim && !mBubble)
             {
                 mCurrAnim = mTheme.random(ANIM_NORMAL);
             }
@@ -393,7 +554,9 @@ void Amor::restack()
         return;
     }
 
-//    kdDebug(10000) << "restacking" << endl;
+#ifdef DEBUG_AMOR
+    kdDebug(10000) << "restacking" << endl;
+#endif
 
     Window sibling = mTargetWin;
     Window dw, parent = None, *wins;
@@ -440,12 +603,14 @@ void Amor::slotMouseClicked(const QPoint &pos)
 
     if (!mMenu)
     {
+        KHelpMenu* help = new KHelpMenu(0, KGlobal::instance()->aboutData(), false);
+        KPopupMenu* helpMnu = help->menu();
         mMenu = new KPopupMenu();
-	mMenu->insertTitle("Amor"); // I really don't want this i18n'ed
-        mMenu->insertItem(SmallIcon("configure"),i18n("&Options..."), this, SLOT(slotConfigure()));
-        mMenu->insertItem(i18n("&About"), this, SLOT(slotAbout()));
+        mMenu->insertTitle("Amor"); // I really don't want this i18n'ed
+        mMenu->insertItem(SmallIcon("configure"), i18n("&Configure..."), this, SLOT(slotConfigure()));
         mMenu->insertSeparator();
-        mMenu->insertItem(SmallIcon("exit"),i18n("&Quit"), kapp, SLOT(quit()));
+        mMenu->insertItem(SmallIcon("help"), i18n("&Help"), helpMnu);
+        mMenu->insertItem(SmallIcon("exit"), i18n("&Quit"), kapp, SLOT(quit()));
     }
 
     mMenu->exec(pos);
@@ -466,6 +631,8 @@ void Amor::slotCursorTimeout()
     QPoint diff = currPos - mCursPos;
     time_t now = time(0);
 
+    if (mForceHideAmorWidget) return; // we're hidden, do nothing
+
     if (abs(diff.x()) > 1 || abs(diff.y()) > 1)
     {
 	if (mState == Sleeping)
@@ -478,8 +645,9 @@ void Amor::slotCursorTimeout()
     }
     else if (mState != Sleeping && now - mActiveTime > SLEEP_TIMEOUT)
     {
-	// The next animation will become sleeping
-	mState = Sleeping;
+	// GP: can't go to sleep if there are tips in the queue
+	if (mTipsQueue.isEmpty())
+	    mState = Sleeping;	// The next animation will become sleeping
     }
 }
 
@@ -491,41 +659,39 @@ void Amor::slotTimeout()
 {
     if ( mForceHideAmorWidget )
         return;
+
     if (!mTheme.isStatic())
 	mPosition += mCurrAnim->movement();
     mAmor->setPixmap(mCurrAnim->frame());
-    mAmor->move(mPosition + mTargetRect.x() - mCurrAnim->hotspot().x(),
-                 mTargetRect.y() - mCurrAnim->hotspot().y() + mConfig.mOffset);
+    mAmor->move(mTargetRect.x() + mPosition - mCurrAnim->hotspot().x(),
+                 mTargetRect.y() - mCurrAnim->hotspot().y() + (!mInDesktopBottom?mConfig.mOffset:0));
     if (!mAmor->isVisible())
     {
         mAmor->show();
         restack();
     }
 
-    // At the start of a base animation, we can randomly display
-    // a helpful tip.
-    if (mCurrAnim == mBaseAnim && mCurrAnim->frameNum() == 0)
+    if (mCurrAnim == mBaseAnim && mCurrAnim->validFrame())
     {
-	if ( !mTipText.isEmpty() && mConfig.mAppTips )
-	{
-	    showBubble( mTipText );
-	    mTipText = QString();
-	}
-	else if (kapp->random()%TIP_FREQUENCY == 1 && mConfig.mTips &&
-		( !mBubble || !mBubble->isVisible() ) )
+	// GP: Application tips/messages can be shown in any frame number; amor tips are 
+	// only displayed on the first frame of mBaseAnim (the old way of doing this).
+	if ( !mTipsQueue.isEmpty() && !mBubble &&  mConfig.mAppTips)
+	    showBubble();
+	else if (kapp->random()%TIP_FREQUENCY == 1 && mConfig.mTips && !mBubble && !mCurrAnim->frameNum())
         {
-            showBubble(mTips.tip());
+	    mTipsQueue.enqueue(new QueueItem(QueueItem::Tip, mTips.tip())); 
+	    showBubble();
         }
     }
 
     if (mTheme.isStatic())
-	mTimer->start(((mState == Normal) || (mState == Sleeping)) ? 10000 : 100, true);
+	mTimer->start((mState == Normal) || (mState == Sleeping) ? 1000 : 100, true);
     else
 	mTimer->start(mCurrAnim->delay(), true);
 
     if (!mCurrAnim->next())
     {
-	if ( mBubble && mBubble->isVisible() )
+	if ( mBubble )
 	    mCurrAnim->reset();
 	else
 	    selectAnimation(mState);
@@ -549,7 +715,7 @@ void Amor::slotConfigure()
     mAmorDialog->show();
 }
 
-//---------------------------------------------------------------------------
+//--------------------------------------------------------------------------
 //
 // Configuration changed.
 //
@@ -569,7 +735,7 @@ void Amor::slotOffsetChanged(int off)
     if (mCurrAnim->frame())
     {
         mAmor->move(mPosition + mTargetRect.x() - mCurrAnim->hotspot().x(),
-                 mTargetRect.y() - mCurrAnim->hotspot().y() + mConfig.mOffset);
+                 mTargetRect.y() - mCurrAnim->hotspot().y() + (!mInDesktopBottom?mConfig.mOffset:0));
     }
 }
 
@@ -581,7 +747,9 @@ void Amor::slotAbout()
 {
     QString about = i18n("Amor Version %1\n\n").arg(AMOR_VERSION) +
                 i18n("Amusing Misuse Of Resources\n\n") +
-                i18n("Copyright (c) 1999 Martin R. Jones <mjones@kde.org>\n") +
+                i18n("Copyright (c) 1999 Martin R. Jones <mjones@kde.org>\n\n") +
+		i18n("Original Author: Martin R. Jones <mjones@kde.org>\n") +
+		i18n("Current Maintainer: Gerardo Puga <gpuga@gioia.ing.unlp.edu.ar>\n" ) +
                 "\nhttp://www.powerup.com.au/~mjones/amor/";
     KMessageBox::about(0, about, i18n("About Amor"));
 }
@@ -600,7 +768,7 @@ void Amor::slotWidgetDragged( const QPoint &delta, bool release )
 	else if (mCurrAnim->totalMovement() + newPosition < 0)
 	    newPosition = -mCurrAnim->totalMovement();
 	mPosition = newPosition;
-        mAmor->move(mPosition + mTargetRect.x() - mCurrAnim->hotspot().x(),
+        mAmor->move(mTargetRect.x() + mPosition - mCurrAnim->hotspot().x(),
                  mAmor->y());
 
 	if ( mTheme.isStatic() && release ) {
@@ -621,7 +789,9 @@ void Amor::slotWidgetDragged( const QPoint &delta, bool release )
 //
 void Amor::slotWindowActivate(WId win)
 {
-//    kdDebug(10000) << "Window activated:" << win << endl;
+#ifdef DEBUG_AMOR
+    kdDebug(10000) << "Window activated:" << win << endl;
+#endif
 
     mTimer->stop();
     mNextTarget = win;
@@ -642,7 +812,7 @@ void Amor::slotWindowActivate(WId win)
         // We are setting focus to a new window
         if (mState != Focus )
 	    selectAnimation(Focus);
-        mTimer->start(0, true);
+	mTimer->start(0, true);
     }
     else
     {
@@ -658,7 +828,10 @@ void Amor::slotWindowActivate(WId win)
 //
 void Amor::slotWindowRemove(WId win)
 {
-//    kdDebug(10000) << "Window removed" << endl;
+#ifdef DEBUG_AMOR
+    kdDebug(10000) << "Window removed" << endl;
+#endif
+
     if (win == mTargetWin)
     {
         // This is an active event that affects the target window
@@ -676,7 +849,9 @@ void Amor::slotWindowRemove(WId win)
 //
 void Amor::slotStackingChanged()
 {
-//    kdDebug(10000) << "Stacking changed" << endl;
+#ifdef DEBUG_AMOR
+    kdDebug(10000) << "Stacking changed" << endl;
+#endif
 
     // This is an active event that affects the target window
     time(&mActiveTime);
@@ -690,9 +865,8 @@ void Amor::slotStackingChanged()
 //
 // Properties of a window changed
 //
-void Amor::slotWindowChange(WId win)
+void Amor::slotWindowChange(WId win, const unsigned long * properties)
 {
-//    kdDebug(10000) << "Window changed" << endl;
 
     if (win != mTargetWin)
     {
@@ -707,17 +881,46 @@ void Amor::slotWindowChange(WId win)
     if (info.isIconified() ||
         info.mappingState == NET::Withdrawn)
     {
-//        kdDebug(10000) << "Iconic" << endl;
+#ifdef DEBUG_AMOR
+        kdDebug(10000) << "Target window iconified" << endl;
+#endif
+
         // The target window has been iconified
         selectAnimation(Destroy);
         mTargetWin = None;
         mTimer->stop();
         mTimer->start(0, true);
+
+	return;
     }
-    else
+    
+    if (properties[0] & NET::WMGeometry) 
     {
-        // The size or position of the window has changed.
-        mTargetRect = info.frameGeometry;
+#ifdef DEBUG_AMOR
+        kdDebug(10000) << "Target window moved or resized" << endl;
+#endif
+
+        QRect newTargetRect = KWin::windowInfo(mTargetWin).frameGeometry();
+
+	// if the change in the window caused the animation to fall 
+	// out of the working area of the desktop, or if the animation 
+	// didn't fall in the working area before but it does now, then
+	//  refocus on the current window so that the animation is 
+	// relocated.
+	QRect desktopArea = mWin->workArea();
+
+	bool fitsInWorkArea = !(newTargetRect.y() - mCurrAnim->hotspot().y() + mConfig.mOffset < desktopArea.y()); 
+	if ((!fitsInWorkArea && !mInDesktopBottom) || (fitsInWorkArea && mInDesktopBottom))
+	{
+	    mNextTarget = mTargetWin;
+	    selectAnimation(Blur);
+	    mTimer->start(0, true);
+
+	    return;
+	}
+
+	if (!mInDesktopBottom)
+	    mTargetRect = newTargetRect;
 
         // make sure the animation is still on the window.
         if (mCurrAnim->frame())
@@ -737,13 +940,13 @@ void Amor::slotWindowChange(WId win)
             else if (mPosition > mTargetRect.width() -
                     (mCurrAnim->frame()->width() - mCurrAnim->hotspot().x()))
             {
-                mPosition = mTargetRect.width() -
-                    (mCurrAnim->frame()->width() - mCurrAnim->hotspot().x());
+                mPosition = mTargetRect.width() - (mCurrAnim->frame()->width() - mCurrAnim->hotspot().x());
             }
-            mAmor->move(mPosition + mTargetRect.x() - mCurrAnim->hotspot().x(),
-                     mTargetRect.y() - mCurrAnim->hotspot().y() +
-                     mConfig.mOffset);
+            mAmor->move(mTargetRect.x() + mPosition - mCurrAnim->hotspot().x(),
+                     mTargetRect.y() - mCurrAnim->hotspot().y() + (!mInDesktopBottom?mConfig.mOffset:0));
         }
+
+	return;
     }
 }
 
@@ -751,9 +954,17 @@ void Amor::slotWindowChange(WId win)
 //
 // Changed to a different desktop
 //
-void Amor::slotDesktopChange(int)
+void Amor::slotDesktopChange(int desktop)
 {
-//    kdDebug(10000) << "Desktop change" << endl;
+    // GP: signal currentDesktopChanged seems to be emitted even if you 
+    // change to the very same desktop you are in.
+    if (mWin->currentDesktop() == desktop)
+	return;
+
+#ifdef DEBUG_AMOR
+    kdDebug(10000) << "Desktop change" << endl;
+#endif
+
     mNextTarget = None;
     mTargetWin = None;
     selectAnimation( Normal );
@@ -761,6 +972,39 @@ void Amor::slotDesktopChange(int)
     mAmor->hide();
 }
 
+// GP ===========================================================================
+
+void Amor::slotBubbleTimeout()
+{
+    // has the queue item been displayed for long enough?
+    QueueItem *first = mTipsQueue.head();
+#ifdef DEBUG_AMOR
+    if (!first)	kdDebug(10000) << "Amor::slotBubbleTimeout(): empty queue!" << endl;
+#endif
+    if ((first->time() > BUBBLE_TIME_STEP) && (mBubble->isVisible()))
+    {
+    	first->setTime(first->time() - BUBBLE_TIME_STEP);
+	mBubbleTimer->start(BUBBLE_TIME_STEP, true);
+	return;
+    }
+
+    // do not do anything if the mouse pointer is in the bubble
+    if (mBubble->mouseWithin())
+    {
+	first->setTime(500);		// show this item for another 500ms
+	mBubbleTimer->start(BUBBLE_TIME_STEP, true);
+	return;	
+    }
+    
+    // are there any other tips pending?
+    if (mTipsQueue.count() > 1)
+    {
+	mTipsQueue.dequeue();
+	showBubble();	// shows the next item in the queue
+    } else
+	hideBubble(true); // hideBubble calls dequeue() for itself.
+}
+
 //===========================================================================
 
 AmorSessionWidget::AmorSessionWidget()
