@@ -1,10 +1,29 @@
-//---------------------------------------------------------------------------
-//
-// amor.h
-//
-// Copyright (c) 1999 Martin R. Jones <mjones@kde.org>
-//
+/* amor.h
+**
+** Copyright (c) 1999 Martin R. Jones <mjones@kde.org>
+**
+*/
 
+/*
+** This program is free software; you can redistribute it and/or modify
+** it under the terms of the GNU General Public License as published by
+** the Free Software Foundation; either version 2 of the License, or
+** (at your option) any later version.
+**
+** This program is distributed in the hope that it will be useful,
+** but WITHOUT ANY WARRANTY; without even the implied warranty of
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+** GNU General Public License for more details.
+**
+** You should have received a copy of the GNU General Public License
+** along with this program in a file called COPYING; if not, write to
+** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
+** MA 02111-1307, USA.
+*/
+
+/*
+** Bug reports and questions can be sent to kde-devel@kde.org
+*/
 #ifndef AMOR_H
 #define AMOR_H
 
@@ -13,6 +32,7 @@
 #endif
 
 #include <qwidget.h>
+#include <qptrqueue.h>
 
 #include "amoranim.h"
 #include "amortips.h"
@@ -27,6 +47,25 @@ class QTimer;
 class KWinModule;
 class KPopupMenu;
 
+class QueueItem {
+public:
+
+    enum itemType { Talk , Tip };
+    
+    QueueItem(itemType ty, QString te, int ti = -1);
+
+    itemType	type() { return iType; }
+    QString	text() { return iText; };
+    int		time() { return iTime; };
+
+    void	setTime(int newTime) { if (iTime > 0) iTime = newTime; };
+
+private:
+    itemType	iType;
+    QString 	iText;
+    int		iTime;
+};
+
 //---------------------------------------------------------------------------
 //
 // Amor handles window manager input and animation selection and updates.
@@ -39,6 +78,8 @@ public:
     virtual ~Amor();
 
     virtual void showTip(QString tip);
+    virtual void showMessage(QString message);
+    virtual void showMessage(QString message, int msec);
     virtual void screenSaverStopped();
     virtual void screenSaverStarted();
 
@@ -48,7 +89,7 @@ public slots:
     void slotWindowActivate(WId);
     void slotWindowRemove(WId);
     void slotStackingChanged();
-    void slotWindowChange(WId);
+    void slotWindowChange(WId, const unsigned long * properties);
     void slotDesktopChange(int);
 
 protected slots:
@@ -61,7 +102,9 @@ protected slots:
     void slotAbout();
     void slotWidgetDragged( const QPoint &delta, bool release );
     void restack();
-    void hideBubble();
+    void hideBubble(bool forceDequeue = false);
+
+    void slotBubbleTimeout();
 
 protected:
     enum State { Focus, Blur, Normal, Sleeping, Waking, Destroy };
@@ -69,7 +112,7 @@ protected:
     bool readConfig();
     void readGroupConfig(KConfigBase &config, QPtrList<AmorAnim> &animList,
                             const char *seq);
-    void showBubble(const QString& msg);
+    void showBubble();
     AmorAnim *randomAnimation(QPtrList<AmorAnim> &animList);
     void selectAnimation(State state=Normal);
     void active();
@@ -88,7 +131,7 @@ private:
     QTimer           *mTimer;      // Frame timer
     QTimer           *mCursorTimer;// Cursor timer
     QTimer           *mStackTimer; // Restacking timer
-    QTimer           *mBubbleTimer;// Bubble tip timer
+    QTimer           *mBubbleTimer;// Bubble tip timer (GP: I didn't create this one, it had no use when I found it)
     AmorDialog       *mAmorDialog; // Setup dialog
     KPopupMenu       *mMenu;       // Our menu
     time_t           mActiveTime;  // The time an active event occurred
@@ -96,9 +139,13 @@ private:
     QString          mTipText;     // Text to display in a bubble when possible
     AmorBubble       *mBubble;     // Text bubble
     AmorTips         mTips;        // Tips to display in the bubble
+    bool	     mInDesktopBottom; // the animation is not on top of the 
+				       // title bar, but at the bottom of the desktop
 
     AmorConfig       mConfig;      // Configuration parameters
     bool             mForceHideAmorWidget;
+
+    QPtrQueue<QueueItem> mTipsQueue; // GP: tips queue
 };
 
 //---------------------------------------------------------------------------
