@@ -1,10 +1,29 @@
-//---------------------------------------------------------------------------
-//
-// amorbubble.cpp
-//
-// Copyright (c) 1999 Martin R. Jones <mjones@kde.org>
-//
+/* amorbubble.cpp
+**
+** Copyright (c) 1999 Martin R. Jones <mjones@kde.org>
+**
+*/
 
+/*
+** This program is free software; you can redistribute it and/or modify
+** it under the terms of the GNU General Public License as published by
+** the Free Software Foundation; either version 2 of the License, or
+** (at your option) any later version.
+**
+** This program is distributed in the hope that it will be useful,
+** but WITHOUT ANY WARRANTY; without even the implied warranty of
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+** GNU General Public License for more details.
+**
+** You should have received a copy of the GNU General Public License
+** along with this program in a file called COPYING; if not, write to
+** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
+** MA 02111-1307, USA.
+*/
+
+/*
+** Bug reports and questions can be sent to kde-devel@kde.org
+*/
 #include "amorbubble.h"
 #include "amorbubble.moc"
 #include <qpainter.h>
@@ -33,6 +52,9 @@ AmorBubble::AmorBubble()
     mBrowser = new KTextBrowser( this );
     mBrowser->setFrameStyle( QFrame::NoFrame | QFrame::Plain );
     mBrowser->setMargin( 0 );
+
+    mBrowser->setWrapPolicy(QTextEdit::AtWordOrDocumentBoundary); // too long to fit in one line?
+
     QColorGroup clgrp = mBrowser->colorGroup();
     clgrp.setColor(QColorGroup::Text, Qt::black);
     //Laurent QTextBrowser didn't have this function FIX me
@@ -48,8 +70,7 @@ AmorBubble::AmorBubble()
     for (it = icons.begin(); it != icons.end(); ++it)
 	mBrowser->mimeSourceFactory()->addFilePath(*it);
 
-    mBubbleTimer = new QTimer(this);
-    connect(mBubbleTimer, SIGNAL(timeout()), SLOT(hide()));
+    mMouseWithin = false;
 }
 
 //---------------------------------------------------------------------------
@@ -74,7 +95,6 @@ void AmorBubble::setMessage(const QString& message)
     mBrowser->setGeometry( 0, 0, 250, 1000 );
     mBrowser->setText( mMessage );
     calcGeometry();
-    mBubbleTimer->start(BUBBLE_TIMEOUT + message.length() * 30, TRUE);
 }
 
 //---------------------------------------------------------------------------
@@ -167,7 +187,8 @@ void AmorBubble::drawBubble(QPainter &p)
         pointArray.translate(0, height() - BORDER_SIZE - ARROW_HEIGHT / 2);
     }
 
-    p.drawRoundRect(left, 0, width() - ARROW_WIDTH, height(), 10, 20);
+//    p.drawRoundRect(left, 0, width() - ARROW_WIDTH, height(), 10, 20);
+    p.drawRect(left, 0, width() - ARROW_WIDTH, height());
 
     QPen pen(p.pen());
     p.setPen(NoPen);
@@ -204,15 +225,22 @@ bool AmorBubble::eventFilter( QObject *, QEvent *e )
 {
     switch ( e->type() )
     {
+
+// GP	case QEvent::Enter:
+// GP	    mBubbleTimer->stop();
+// GP	    break;
+// GP	case QEvent::Leave:
+// GP	    if ( isVisible() )
+// GP		mBubbleTimer->start( 1000, true );
+// GP	    break;
 	case QEvent::Enter:
-	    mBubbleTimer->stop();
+	    mMouseWithin = true;
 	    break;
 	case QEvent::Leave:
-	    if ( isVisible() )
-		mBubbleTimer->start( 1000, true );
+	    mMouseWithin = false;
 	    break;
 	case QEvent::MouseButtonRelease:
-	    hide();
+	    hide();				// GP This is the only reason a bubble might posibly be created but hidden
 	    break;
 	default:
 	    break;
