@@ -1,10 +1,29 @@
-//---------------------------------------------------------------------------
-//
-// amordialog.h
-//
-// Copyright (c) 1999 Martin R. Jones <mjones@kde.org>
-//
+/* amordialog.h
+**
+** Copyright (c) 1999 Martin R. Jones <mjones@kde.org>
+**
+*/
 
+/*
+** This program is free software; you can redistribute it and/or modify
+** it under the terms of the GNU General Public License as published by
+** the Free Software Foundation; either version 2 of the License, or
+** (at your option) any later version.
+**
+** This program is distributed in the hope that it will be useful,
+** but WITHOUT ANY WARRANTY; without even the implied warranty of
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+** GNU General Public License for more details.
+**
+** You should have received a copy of the GNU General Public License
+** along with this program in a file called COPYING; if not, write to
+** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
+** MA 02111-1307, USA.
+*/
+
+/*
+** Bug reports and questions can be sent to kde-devel@kde.org
+*/
 #ifndef AMORDIALOG_H
 #define AMORDIALOG_H
 
@@ -44,9 +63,11 @@ protected slots:
     void slotHighlighted(int);
     void slotOnTop(bool);
     void slotRandomTips(bool);
+    void slotRandomTheme(bool);
     void slotApplicationTips(bool);
     void slotOffset(int);
     void slotOk();
+    void slotApply();
     void slotCancel();
 
 protected:
